//------------------------------------------------------------------------//
//                                                                        //
// File Name   : Dialog.cpp                                               //
// Description : This program is a CPP version of dialog.c program        //
//               It shows how to replace the Switch statement in the      //
//               call back function of a window or a dialog.              //
//		 It uses MFT_DEF, MFT_INIT macros.  			  //
//               The defination of these macros are present in MFT.H      //
//                                                                        //
//               MFT_DEF(className, CallBackProc) defines the type        //
//		 of a structure,                                          //
//		 which holds the Message and the pointer to the function, //
//               which is a callback function for that message. This      //
//		 must be declared inside your window class defination,    //
//		 which is derived from the class "Window" or "Dialog" etc.//
//		 e.g WM_CREATE is the message, wmCreate() is the function //
//		 which acts as a call back function for it.               //
//                                                                        //
//               MFT_INIT(className, CallBackProc, DefaultCallBackProc)   //
//		 initializes the structure  with the                      //
//		 the messages and corresponding functions. This must be   //
//		 declared out side the class defination.                  //
//               eg, CallBackProc = WndProc() or DlgProc();               //
//		 DefaultCallBackProc = WinDefWindowProc(), WinDefDlgProc()//
//               className = WhereDlg, MainWindow etc.                    //
//                                                                        //
//               This program demonstrates how to associate a pop-up      //
//               window with a standard window.  The pop-up window        //
//               is defined as a dialog template in a resource file.      //
//                                                                        //
// Concepts    : This program displays a standard window.  The action bar //
//               contains a single choice:  Options.  The Options         //
//               pull-down contains a choice that displays a pop-up       //
//               window which contains an entry field, and OK and Cancel  //
//               pushbuttons.  When the user types a place name in the    //
//               entry field and selects OK, the pop-up window            //
//               disappears, and the place name is displayed at the end   //
//               of the text "You live in".                               //
//                                                                        //
// API's       :                                                          //
// -------------                                                          //
// GpiCharStringAt       WinDestroyWindow    WinInvalidateRegion          //
// GpiSetColor           WinDismissDlg       WinLoadString                //
// WinBeginPaint         WinDispatchMsg      WinPostMsg                   //
// WinCreateMsgQueue     WinDlgBox           WinQueryWindowText           //
// WinCreateStdWindow    WinEndPaint         WinRegisterClass             //
// WinDefDlgProc         WinFillRect         WinSendMsg                   //
// WinDefWindowProc      WinGetMsg           WinTerminate                 //
// WinDestroyMsgQueue    WinInitialize       WinWindowFromID              //
//                                                                        //
// CopyRight(c) 1993, Borland International Inc.                          //
//                                                                        //
//------------------------------------------------------------------------//

#define INCL_PM                         // Define to "switch on" PM includes
					// in "os2.h"
#include <os2.h>                        // PM includes
#include <string.h>                     // String functions
#include <stdlib.h>
#include "dialog.h"                     // Resource symbolic identifiers
#include <stdio.h>                      // vsprintf() prototype
#include <stdarg.h>

#include "mft.h"                        // message function table
#include "win.h"                        // Window class defination
#include "dlg.h"                        // Dialog class defination

#define INCL_WINSYS                     // Selectively include
#define INCL_WINFRAMEMGR                // relevant parts of
#define INCL_WINMENUS                   // the PM header file
#define INCL_WINDIALOGS
#define INCL_GPI


#define LENGTH_STRING  33               // Length of string
#define RETURN_ERROR    1               // return value for exit
#define BEEP_WARN_FREQ      60          // frequency of warning beep
#define BEEP_WARN_DUR      100          // duration of warning beep

#undef NULL
#define NULL 0L

// Prototype for window callback function
// This should be the entry point for all window classes
// These general call back functions cannot be declared inside a class
// declaration, because all the functions declared inside a class has
// "this" as 1st parameter. You will get a type mismatch error, if you
// say WinRegisterClass(.... WinCallBack....), if WinCallBack is in a class.
// WinCallBack inturn calls the actual callback function(WndProc()).

MRESULT EXPENTRY _export WinCallBack(HM12);
MRESULT EXPENTRY _export DlgCallBack(HM12);
static VOID SetSysMenu(HWND hwndDlg);
void adjustProdInfoDlg(HM12);
void CenterDialog(HWND hDlg);

class CallBack{
public:
   static Window   *winThis;            // Used to hold the "this" pointer
					// that _export WinCallBack(must be
					// global)uses to vector
					// to the correct member function
					// WndProc(dispatcher)
   static Dialog   *dlgThis;            // Used to hold the "this" pointer
					// that _export DlgCallBack(must be
					// global) uses to vector
					// to the correct member function
					// DlgProc(diapatcher)
};

Window *CallBack::winThis = NULL;
Dialog *CallBack::dlgThis = NULL;

//----------------------------class PMApp{}-------------------------------
// Class main is a container of program global functions and variables

class PMApp
{
public:
    static HAB      hab;
    static HMQ      hmq;
    static CHAR     szLocation[LENGTH_STRING];   // To hold text entered by user
    static CHAR     szResponse[LENGTH_STRING];   // To hold "You live in"
    static BOOL     bComplete ;                  // Switch for first time through
    PMApp();                            // Constructor
    ~PMApp();                           // Destructor

    // Standard PM Initilization sequence
    static BOOL Initialize(void);

    // Standard PM Message Loop
    static BOOL Run(void);

    // Standard PM Termination sequence
    static void Terminate(void);

    static MRESULT EXPENTRY ProdInfoDlgProc(HM12);
    static VOID DisplayMessage( char *str, ... );

};

// Declare and initialize all globals in PMApp
HAB    PMApp::hab;
HMQ    PMApp::hmq;
CHAR   PMApp::szLocation[LENGTH_STRING] = "";   // To hold text entered by user
CHAR   PMApp::szResponse[LENGTH_STRING] = "";   // To hold "You live in"
BOOL   PMApp::bComplete = FALSE;                // Switch for first time through


//-----------------------class MainWindow{}------------------------------

class MainWindow : public Window
{
private:
    static char szClassName[14];
    static BOOL initialized;      // tracks window is registered, or not

public:
   void Register(void);
   MainWindow   (void);                // Constructor
   ~MainWindow  (void);                // Destructor

   MFT_DEF(MainWindow, WndProc)        // Message Function defination for MainWindow

   MRESULT wmCreate (HM12);
   MRESULT wmCommand(HM12);
   MRESULT wmPaint  (HM12);
   MRESULT wmClose  (HM12);

};

// This is the Message Function Table.(MFT) initialization
// This is only for the messages not for the mp1 and mp2 params.

MFT_INIT(MainWindow, WndProc, WinDefWindowProc)
//-------- Message --- callBack -----------------
	{ WM_COMMAND, wmCommand},
	{ WM_CREATE,  wmCreate },
	{ WM_PAINT,   wmPaint  },
	{ WM_CLOSE,   wmClose  },
MFT_INIT_END

char MainWindow::szClassName[14] = "MyWindow";  // Must be unique in the app
BOOL MainWindow::initialized = FALSE;

//--------------------------class WhereDlg{}-------------------------------

class WhereDlg : public Dialog
{
private:
    BOOL initialized;
public:
   WhereDlg   (void){}               // Constructor
   ~WhereDlg  (void)                 // Destructor
   {
	initialized = FALSE;
	CallBack::dlgThis = NULL;
   }

   WhereDlg(HWND hwndParent1,
	 HWND hwndOwner1,
	 HMODULE Resource1,
	 ULONG ulDlgid1,
	 PVOID pCreateParams1):Dialog(hwndParent1,
				      hwndOwner1,
				      Resource1,
				      ulDlgid1,
				      pCreateParams1)
    {
    initialized = TRUE;
    CallBack::dlgThis = this;   // Stuff "this" pointer in global dlgThis
				// for use by _export DlgCallBack.
				// It was assigned as a
				// call back function by WinDlgBox().
				// dlgThis is used to call
				// WhereDlg::DlgProc (pure virtual
				// dispatcher, defined as a macro in dlg.h)
    }

   Run();

   MFT_DEF(WhereDlg, DlgProc)     // Message Function defination for WhereDlg

   MRESULT wmInitDlg(HM12);
   MRESULT wmCommand(HM12);

};

// This is the Message Function Table.(MFT) initialization
// This is only for the messages not for the mp1 and mp2 params.

MFT_INIT(WhereDlg, DlgProc, WinDefDlgProc)
//-------- Message --- callBack -----------------
	{ WM_COMMAND, wmCommand },
	{ WM_INITDLG, wmInitDlg },
MFT_INIT_END

//--------------------------class ProductDlg{}-------------------------------

class ProductDlg : public Dialog
{
private:
    BOOL initialized;
public:
   ProductDlg   (void){}               // Constructor
   ~ProductDlg  (void)                 // Destructor
   {
	initialized = FALSE;
	CallBack::dlgThis = NULL;
   }

   ProductDlg(HWND hwndParent1,
	 HWND hwndOwner1,
	 HMODULE Resource1,
	 ULONG ulDlgid1,
	 PVOID pCreateParams1):Dialog(hwndParent1,
				      hwndOwner1,
				      Resource1,
				      ulDlgid1,
				      pCreateParams1)
    {
    initialized = TRUE;
    CallBack::dlgThis = this;	// Stuff "this" pointer in global dlgThis
				// for use by _export DlgCallBack.
				// It was assigned as a
				// call back function by WinDlgBox().
				// dlgThis is used to call
				// ProductDlg::DlgProc (pure virtual
				// dispatcher, defined as a macro in dlg.h)
    }

   Run();
   void adjustProdInfoDlg(HM12);
   MFT_DEF(ProductDlg, DlgProc)     // Message Function defination for ProductDlg

   MRESULT wmInitDlg(HM12);
   MRESULT wmCommand(HM12);

};

// This is the Message Function Table.(MFT) initialization
// This is only for the messages not for the mp1 and mp2 params.

MFT_INIT(ProductDlg, DlgProc, WinDefDlgProc)
//-------- Message --- callBack -----------------
	{ WM_COMMAND, wmCommand },
	{ WM_INITDLG, wmInitDlg },
MFT_INIT_END

//--------------------------End Class Declaration-------------------------

//-------------------------------PMApp::PMApp()---------------------------
PMApp::PMApp()
{
   // Initialize PM first
   if (!PMApp::Initialize())
   {
      PMApp::DisplayMessage("Could Not Initialize");
      exit(RETURN_ERROR);
   }
}

//-------------------------PMApp::~PMApp()-------------------------------

PMApp::~PMApp()
{
   PMApp::Terminate();
}

//-------------------------PMApp::Initialize()----------------------------
// Standard PM Initilization sequence

BOOL PMApp::Initialize(void) {
  hab = WinInitialize( 0 );
  if(!hab)
  {
    // no message box can displayed here, beep and exit
    DosBeep(BEEP_WARN_FREQ, BEEP_WARN_DUR);
    exit(RETURN_ERROR);
  }

  hmq = WinCreateMsgQueue( hab, 0 );
  if(!hmq)
  {
    DosBeep(BEEP_WARN_FREQ, BEEP_WARN_DUR);
    WinTerminate(hab);
    return(RETURN_ERROR);
  }

  ProductDlg productDialog(
			HWND_DESKTOP, 	       // Place anywhere on desktop
			HWND_DESKTOP,  	       // Owned by desk top
			(HMODULE)0,    	       // Module handle
			ID_PRODINFO,   	       // Dialog identifier in resource
			NULL           	       // Initialization data
			);

  if(!productDialog.Run()){
    DosBeep(BEEP_WARN_FREQ, BEEP_WARN_DUR);
    WinTerminate(hab);
    return(0);                                  //Failure
  }
  return(TRUE);                        		// Success
}

//----------------------PMApp::Run()------------------------------------
// Standard PM Message Loop

BOOL PMApp::Run(void)
{
    BOOL rc;
    QMSG qmsg;

    while((rc = (BOOL)WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))!=FALSE)
	    WinDispatchMsg(hab,&qmsg);

    return rc;
}

//----------------------PMApp::Terminate()------------------------------
// Standard PM Termination sequence

void PMApp::Terminate(void)
{
	WinDestroyMsgQueue( hmq );
	WinTerminate( hab );
}

//----------------------MainWindow::MainWindow()------------------------
// Contstructor

MainWindow::MainWindow(void)
{

    if (initialized == FALSE) Register();
    ULONG flCreate = FCF_STANDARD;
    CallBack::winThis = this;   // Stuff "this" pointer in global
				// for use by _export WinCallBack.
				// _export WinCallBack is called
				// directly as a result of
				// WndCreateStdWindow and will
				// put this in the PM window
				// record and use it to vector to
				// MainWindow::WndProc(dispatcher, gets
				// rid of the switch statement, defined as
				// a macro in win.h)

    hwndFrame = WinCreateStdWindow(
       HWND_DESKTOP,            // Desktop window is parent
       NULL,                    // not displayed, for less flickering
       &flCreate,
       (PSZ)szClassName,
       (PSZ)"",
       0,
       (HMODULE)NULL,           // Resource is in .EXE file
       ID_MAINWND,              // Frame window identifier
       &hwndClient
       );
    if(hwndFrame == (HWND)NULL)
    {
	PMApp::DisplayMessage("Could Not Create Main Window");
	exit(RETURN_ERROR);
    }

    // Set title bar fo window
    WinSetWindowText(hwndFrame, (PSZ)"Dialog - Sample Program");
    if (!WinSetWindowPos( hwndFrame,
		HWND_TOP,            // window at position 20, 100,
		(WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN) - (400))/2,
		(WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN) - (300))/2,
		400, 300,   // and size 400, 300.
		// Flags set to show and activate frame window
		SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW
		)){
	PMApp::DisplayMessage("Could Not Position Main Window");
	exit(RETURN_ERROR);
    }

    WinSetActiveWindow(HWND_DESKTOP, hwndFrame); // Move to top

}

//-------------------MainWindow::~MainWindow()----------------------------

MainWindow::~MainWindow(void)
{
	initialized = FALSE;
	WinDestroyWindow(hwndFrame);
}

//-------------------MainWindow::Register()-------------------------------

void MainWindow::Register(void)
{
	if (!WinRegisterClass(
		PMApp::hab,
                (PSZ)szClassName,
		::WinCallBack,          // use global _export WinCallBack.
					// which will vector the
					// call to MainWindow::WndProc
		CS_SIZEREDRAW,
		sizeof(Window *)        // Allocate space for the "this"
					// pointer after the end of the
					// window record PM creates.
					// This is used to vector to
					// MainWindow::WndProc
    ))
    {
	    PMApp::DisplayMessage("Could Not Register Class");
	    exit(RETURN_ERROR);
    }	initialized = TRUE;
}

//----------------------MainWindow::wmCreate()----------------------------

MRESULT MainWindow::wmCreate(HM12)
{
   // Window initialization - load "You live in" string from resource

   WinLoadString( PMApp::hab,
       (HMODULE)NULL,        // Resource is in .EXE file
       ID_RESPONSE,          // String identifier
       sizeof( PMApp::szResponse ), // Size of buffer
       (PSZ)PMApp::szResponse);     // Buffer
   USE_HM12;
   return (MRESULT)FALSE;
}

//----------------------MainWindow::wmCommand()----------------------------

MRESULT MainWindow::wmCommand(HM12)
{
   MRESULT retVal = (MRESULT)FALSE;
   /*
    * When the user chooses "Where..." from the Options pull-down,
    * the application displays a dialog box. WinDlgBox calls the
    * dialog procedure below. WinDlgBox does not return until the
    * dialog procedure returns.
    * WinInvalidateRegion sends a WM_PAINT message.
    */
   switch ( SHORT1FROMMP( mp1 ) )
   {
	case ID_WHERE:{
	  WhereDlg whereDlg( HWND_DESKTOP,        // Place on DESKTOP
			  hwndFrame,
			  (HMODULE)0,          // No resource in EXE
			  (ULONG)ID_MYDIALOG,
			  (PVOID)NULL);
	  whereDlg.Run();                      // brings up the were dlg
	  WinInvalidateRegion( hwnd, NULLHANDLE, FALSE ); // Force a repaint
	  }
	  break;
	case ID_EXITPROGRAM:

	  WinPostMsg( hwnd, WM_CLOSE, NULL, NULL);

	  break;
	default:
	  retVal =  WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }
   return (MRESULT)retVal;  // must be false unless default.
}

//--------------------------MainWindow::wmPaint()-------------------------

MRESULT MainWindow::wmPaint(HM12)
{
   /*
    * Window contents are drawn here. First time through, PMApp::bComplete
    * is FALSE, so window is simply filled with SYSCLR_WINDOW.
    * On subsequent passes, if bComplete has been set to TRUE in the
    * dialog procedure, GpiCharStringAt draws the text.
    */

    HPS    hps;                      // Presentation space handle
    POINTL pt;                       // String screen coordinates
    RECTL  rc;                       // Window rectangle

	hps = WinBeginPaint( hwnd, (HPS)NULL, &rc );
	WinFillRect( hps, &rc, SYSCLR_WINDOW );
	if( PMApp::bComplete)
	{
	  GpiSetColor( hps, CLR_NEUTRAL ); // Text color same as PM
	  pt.x = 50L; pt.y = 50L;         // Coordinates of "You live in"
          GpiCharStringAt( hps, &pt, (LONG)strlen( PMApp::szResponse ), (PSZ)PMApp::szResponse );
	  pt.x = 50L; pt.y = 30L;         // Coordinates of location
          GpiCharStringAt( hps, &pt, (LONG)strlen( PMApp::szLocation ), (PSZ)PMApp::szLocation );
	}
	WinEndPaint( hps );               // Drawing is complete

   USE_HM12;
   return(MRESULT)FALSE;

}

//--------------------------MainWindow::wmClose()-------------------------

MRESULT MainWindow::wmClose(HM12)
{
      WinPostMsg( hwnd, WM_QUIT, NULL, NULL);     // Cause termination
      USE_HM12;
      return(MRESULT)FALSE;
}

//-----------------------WhereDlg::Run()----------------------------------

WhereDlg::Run()
{
    WinDlgBox(  hwndParent,     // Place anywhere on desktop
		hwndOwner,      // Owned by frame
		::DlgCallBack,  // Address of dialog procedure
		Resource,       // Module handle
		ulDlgid,        // Dialog identifier in resource
		NULL); 		// Initialization data
   return(0);
}

//-----------------------WhereDlg::wmInitDlg()----------------------------

MRESULT WhereDlg::wmInitDlg(HM12)
{
       ::SetSysMenu(hwnd);      // remove unused sys. menu items
       ::CenterDialog(hwnd);
       USE_HM12;
       return FALSE;
}

//-----------------------WhereDlg::wmCommand()-----------------------------

MRESULT WhereDlg::wmCommand(HM12)
{
      USE_HM12;
      /*
       * PM sends a WM_COMMAND message when the user presses either
       * the Enter or Escape pushbuttons.
       */
      switch( SHORT1FROMMP( mp1 ) )     // Extract the command value
      {
	case DID_OK:                    // The Enter pushbutton or key.
					// WinQueryWindowText copies the
					// contents of the entry field
					// into szLocation.
	  WinQueryWindowText( WinWindowFromID( hwnd, ID_ENTRYFIELD ),
			      LENGTH_STRING,
                              (PSZ)PMApp::szLocation );
	  PMApp::bComplete = TRUE;       // Set switch so strings are
	  //Falls through   		// drawn in WM_PAINT processing.
	case DID_CANCEL:         	// The Cancel pushbutton or Escape key
	  WinDismissDlg( hwnd, TRUE );  // Removes the dialog box
	  return (MRESULT) FALSE;
	default:
      /*
       * Any event messages that the dialog procedure has not processed
       * come here and are processed by WinDefDlgProc.
       * This call MUST exist in your dialog procedure.
       */
	  return  WinDefDlgProc( hwnd, msg, mp1, mp2 );
      }
}

//--------------------ProductDlg::wmInitDlg()-----------------------------

MRESULT ProductDlg::wmInitDlg(HM12)
{
	::SetSysMenu(hwnd);              // remove unused sys. menu items
	adjustProdInfoDlg(hwnd, msg, mp1, mp2);
	USE_HM12;
	return(FALSE);
}

//--------------------ProductDlg::wmCommand()-----------------------------

MRESULT ProductDlg::wmCommand(HM12)
{
	 // No matter what the command, close the dialog
	 WinDismissDlg(hwnd, TRUE);
	 USE_HM12;
	 return(FALSE);
}

//--------------------ProductDlg::Run()-----------------------------

ProductDlg::Run()
{
  int success;
  success =  WinDlgBox(  hwndParent,      // Place anywhere on desktop
		hwndOwner,      // Owned by frame
		::DlgCallBack,  // Address of dialog procedure
		Resource,       // Module handle
		ulDlgid,        // Dialog identifier in resource
		NULL); 		// Initialization data
  if (!success)
  {
    PMApp::DisplayMessage("Could not Display dialog box");
    return(0);
  }
  return(1);
}

//------------------------ProductDlg::adjustProdInfoDlg()------------------
// This function adjusts the productinfo dialog.
// The position of the bit map in a dialog varies depending on
// the video driver resolution, that is why
// all the other components are positioned with respect to the bit map
// location and size, that makes the dialog look, always the same.

void ProductDlg::adjustProdInfoDlg(HM12)
{
	int width, height;
	const DeltaY = 80; // extra boost in ht. for ID_TXTs and Ok button
	SWP swpTXT1;	// dimension of ID_TXT1 resource.(Demonstrates..)
	SWP swpTXT2;	// dimension of ID_TXT2 resource.(This prog...)
	SWP swpCPR;	// dimension of CopyRight Line.
	SWP swpOk;	// dimension of OK button.
	BITMAPINFOHEADER bih;

	// Load and get the size of Object Bitmap.
	HPS hps = WinGetPS(hwnd);
	HBITMAP  hbmpSplash = GpiLoadBitmap(hps, NULLHANDLE, ID_BITMAP, 0, 0);

	GpiQueryBitmapParameters(hbmpSplash, &bih);
	GpiDeleteBitmap(hbmpSplash);
	WinReleasePS(hps);

	//Get Size of Ok button
	if(!WinQueryWindowPos(WinWindowFromID(hwnd, ID_PRODINFO_OK), &swpOk))
	{
		PMApp::DisplayMessage("Cannot get size for OK button");
		exit(-1);
	}

	//Get Size of CopyRight
	if(!WinQueryWindowPos(WinWindowFromID(hwnd, ID_CPR), &swpCPR))
	{
		PMApp::DisplayMessage("Cannot get size of CopyRight Notice.");
		exit(-1);
	}

	//Get Size of ID_TXT2( This program ... )
	if(!WinQueryWindowPos(WinWindowFromID(hwnd, ID_TXT2), &swpTXT2))
	{
		PMApp::DisplayMessage("Cannot get size of ID_TXT2 resource");
		exit(-1);
	}

	//Get Size of ID_TXT1(Demonstrates ... )
	if(!WinQueryWindowPos(WinWindowFromID(hwnd, ID_TXT1), &swpTXT1))
	{
		PMApp::DisplayMessage("Cannot get size of ID_TXT1 resource");
		exit(-1);
	}

	//Calculate width and ht. of frame dialog.
	width = WinQuerySysValue(HWND_DESKTOP, SV_CXDLGFRAME) * 2 + bih.cx;
	height = WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME) * 2 + 16 +
		WinQuerySysValue(HWND_DESKTOP, SV_CYTITLEBAR) + DeltaY +
		swpOk.cy + bih.cy;

	// Make Frame Size just a little bit larger than Bitmap size
	WinSetWindowPos(hwnd, 0, 0, 0, width, height,	SWP_SIZE);

	// move the OK button to correct position from bottom left.
	WinSetWindowPos
	(
		WinWindowFromID(hwnd, ID_PRODINFO_OK),
		0,
		(width-swpOk.cx)/2,            // center OK button
		WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME)+ 8,
		0,
		0,
		SWP_MOVE
	);

	// Move CopyRight notice to correct position above OK button.
	WinSetWindowPos
	(
		WinWindowFromID(hwnd, ID_CPR),
		0,
		(width-swpCPR.cx)/2,            // center CopyRight
		WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME)+ 8 + 35,
		0,
		0,
		SWP_MOVE
	);

	// Move ID_TXT2(This program is a C++ Conversion of dialog.c.)
	// to correct position above CopyRight.
	WinSetWindowPos
	(
		WinWindowFromID(hwnd, ID_TXT2),
		0,
		(width-swpTXT2.cx)/2,            // center ID_TXT2
		WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME)+ 8 + 35 + 20,
		0,
		0,
		SWP_MOVE
	);

	// Move ID_TXT1(Demonstrates C to C++ conversion)
	// to correct position above ID_TXT2.
	WinSetWindowPos
	(
		WinWindowFromID(hwnd, ID_TXT1),
		0,
		(width-swpTXT1.cx)/2,            // center ID_TXT1
		WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME)+ 8 + 35 + 20 + 20,
		0,
		0,
		SWP_MOVE
	);

	// Sizes and moves the Bitmap inside the frame.
	// Bottom Left corner of frame is 0, 0.
	WinSetWindowPos
	(
		WinWindowFromID(hwnd, ID_BITMAP),
		0,
		WinQuerySysValue(HWND_DESKTOP, SV_CXDLGFRAME),
		WinQuerySysValue(HWND_DESKTOP, SV_CYDLGFRAME)+16+swpOk.cy+DeltaY,
		bih.cx,
		bih.cy,
		SWP_MOVE | SWP_SIZE
	);
	USE_HM12;
	::CenterDialog(hwnd);
}

//--------------------------WinCallBack()----------------------------------

MRESULT EXPENTRY _export WinCallBack(HM12)
{
    MRESULT retVal = (MRESULT)FALSE;
    // Pointer to the c++ window object
    Window *pWindow = (Window *)WinQueryWindowULong(hwnd,0);

    // The pointer pWindow will have an invalid value if the WM_CREATE
    // message has not yet been processed (we respond to the WM_CREATE
    // message by setting the extra bytes to be a pointer to the
    // (C++) object corresponding to the Window identified
    // by hwnd).  The messages that
    // precede WM_CREATE must be processed without using pWindow so we
    // pass them to WinDefWindowProc.

    if (pWindow == 0) {
	    if (msg == WM_CREATE) {
		pWindow = CallBack::winThis;
		WinSetWindowULong(hwnd,0,(ULONG)pWindow);
		retVal = pWindow->WndProc(hwnd,msg,mp1,mp2);
	    }
       else
		retVal = WinDefWindowProc(hwnd,msg,mp1,mp2);
    }
    else
	    retVal =  pWindow->WndProc(hwnd,msg,mp1,mp2);

    return(retVal);
}

//--------------------------DlgCallBack()---------------------------------
// N.B: If this function is going to be used by more than 1 dialog box at
// the same time, you must save the CallBack::dlgThis for different hwnd in
//  a local linked list, and free it when the WM_CLOSE(or whatever
// terminating message) is issued.

MRESULT EXPENTRY _export DlgCallBack(HM12)
{
    MRESULT retVal = (MRESULT)FALSE;
    // Pointer to the c++ window object
    Dialog *pDialog;

    pDialog = CallBack::dlgThis;
    if(pDialog)
    {
	retVal = pDialog->DlgProc(hwnd,msg,mp1,mp2);
    }
    else
	retVal = WinDefDlgProc(hwnd,msg,mp1,mp2);
    return(retVal);
}

//-----------------------------CenterDialog()-----------------------------
// This function places a dialog in the center of the Desktop

void CenterDialog(HWND hDlg)
{
  RECTL r;

  WinQueryWindowRect(hDlg, &r);
  WinSetWindowPos
  (
    hDlg,
    0,
    (WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN) - (r.xRight-r.xLeft))/2,
    (WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN) - (r.yTop-r.yBottom))/2,
    r.xRight - r.xLeft,
    r.yTop - r.yBottom,
    SWP_MOVE
  );
}

//---------------------------------------------------------------
//  Name:   DisplayMessage
//
//  Description : Displays error messages.
//
//  Concepts : Displays a message box containing an error
//             message each time called.
//
//  API's : WinMessageBoxNon
//
//  Parameters   : Same as printf();
//
//  Returns:  Void
//
//---------------------------------------------------------------
#define MAX_KEYSTR_LEN 256
VOID PMApp::DisplayMessage(char *fmt, ...)
{
  /*
   * This routine calls WinMessageBox to dsplay error messages to the
   * user.  It accepts printf like arguments.
   */

char String[MAX_KEYSTR_LEN];
va_list argptr;
int cnt;

   va_start(argptr, fmt);
   cnt = vsprintf(String, fmt, argptr);

   if(cnt > MAX_KEYSTR_LEN){
       WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) "String Buffer Overflow",
	 (PCH) "Messages Dialog Sample", ID_MESSAGEBOX,
	   MB_OK | MB_APPLMODAL | MB_MOVEABLE | MB_ERROR);

		exit(-1);
   }
   va_end(argptr);


  WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) String,
     (PCH) "Error Messages For The Dialog Sample", ID_MESSAGEBOX,
     MB_OK | MB_APPLMODAL | MB_MOVEABLE | MB_ERROR);
}

//----------------------------------------------------------
// Name         : SetSysMenu
//
// Description  : Procedure to remove unselectable items from
//                the window system menu.
//
// Concepts     : This routine determines the number of items
//                in the system menu, loops through those
//                items removing disabled menu items and menu
//                seperators.
//
// API's        : WinMessageBox
//
// Parameters   : None
//
// Return       : None
//
//-------------------------------------------------------------

static VOID SetSysMenu(HWND hwndDlg)
{
  SHORT sMenuItems;
  USHORT usItemid;
  MENUITEM menuItem;
  HWND  hwndSubMenu;

  // Determine the definition of the system menu
  WinSendDlgItemMsg(hwndDlg,(SHORT)FID_SYSMENU, MM_QUERYITEM,
		    MPFROM2SHORT((SHORT)SC_SYSMENU,FALSE),
		    MPFROMP(&menuItem));

  hwndSubMenu = menuItem.hwndSubMenu;

  // Find the number of items in the in the submenu
  sMenuItems = (SHORT)WinSendMsg(hwndSubMenu,
				 MM_QUERYITEMCOUNT,NULL,NULL);

   // Loop through the submenu items and remove disabled
   // menu items and menu separators.

  for (sMenuItems-- ; sMenuItems >= 0 ;sMenuItems-- )
  {
    // Find the item ID for the current position.
    usItemid = (USHORT)WinSendMsg(hwndSubMenu,MM_ITEMIDFROMPOSITION,
			 MPFROMSHORT(sMenuItems), NULL);

    // Query the definition of the current item
    WinSendMsg(hwndSubMenu, MM_QUERYITEM,
	       MPFROM2SHORT(usItemid,FALSE), MPFROMP(&menuItem));

   /*
    * If the menu item is disabled or the item has a style
    * of MIS_SEPARATOR - delete it.
    */
    if ((BOOL)(WinSendMsg(hwndSubMenu, MM_QUERYITEMATTR,
	  MPFROM2SHORT(usItemid,FALSE), (MPARAM)MIA_DISABLED)) ||
	  (menuItem.afStyle & MIS_SEPARATOR))
    {
      WinSendMsg(hwndSubMenu, MM_DELETEITEM,
		MPFROM2SHORT(usItemid,FALSE),NULL);
    }
  } // endfor
}

//------------------------main()------------------------------------------

void cdecl main()
{
	PMApp dialogApp;
	MainWindow MainWnd; // Set up new scope and create an instance
			    // of MainWindow, which causes it to
			    // Register the class and Create the
			    // main window.  When MainWnd goes out
			    // of scope, the destructor will be called
			    // which cleans up with WinDestroyWindow
	dialogApp.Run();
}


